SystemOrganization addCategory: #'AmbientTalkST-Benchmarks'!

BenchmarkGameSuite subclass: #ATBenchmarkGameSuite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!ATBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 1/4/2012 15:09'!
benchATBinaryTrees
	| bench problemSize treeActor benchActor treeFactory result |
	problemSize := self problemSize
			ifNotNil: [ self problemSize ]
			ifNil:    [ BGBinaryTrees defaultProblemSize ].
	
	treeActor := self createActor: 'Trees'.
	benchActor := self createActor: 'BenchExec'.
	
	treeFactory := treeActor adoptObject: BGTreeNodeFactory new.
	
	bench := BGBinaryTrees new.
	bench binaryTreeFactory: treeFactory.
	
	result := benchActor evaluateAsync: [
		bench binarytrees: problemSize to: (TextStream with: String new).
	].
	
	^ result read.! !

!ATBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 1/15/2012 22:14'!
benchATFannkuchRedux
	| problemSize genActor gen benchActor result genClass |
	problemSize := self problemSize
			ifNotNil: [ self problemSize ]
			ifNil:    [ BGFannkuchRedux defaultProblemSize ].

	genActor   := self createActor: 'PermGenRedux'.
	benchActor := self createActor: 'BenchExec'.
	
	(Smalltalk hasClassNamed: #OstBGPermGeneratorRedux)
		ifTrue: [genClass := Smalltalk at: #OstBGPermGeneratorRedux]
		ifFalse:[genClass := BGPermGeneratorRedux].
	
	gen := (genActor evaluateAsync: [ genClass new: problemSize ]) read.
	"For AT we need to make sure it is properly enforced"
	gen := genActor adoptObject: gen.
	
	result := benchActor evaluateAsync: [
		BGFannkuchRedux maxPfannkuchenTo: (TextStream with: String new) with: gen ].
	
	^ result read.
! !

!ATBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 1/15/2012 22:24'!
benchATFasta
	| randomnessActor benchActor bodies out bench ranClass |
	bench := BGFasta new.
	bench n: (self problemSize
						ifNotNil: [ self problemSize ]
						ifNil:    [ BGFasta defaultProblemSize ]).
	out := TextStream with: String new.
	bench out: out.
	
	randomnessActor := self createActor: 'Randomness'.
	benchActor      := self createActor: 'BenchExec'.
	
	ranClass := (Smalltalk hasClassNamed: #OstBGRandomNumber)
		ifTrue: [Smalltalk at: #OstBGRandomNumber]
		ifFalse:[BGRandomNumber].
	
	bench repeatStreamFact: (randomnessActor adoptObject: BGRepeatStreamFactory new).
	
	(benchActor evaluateAsync: [bench oneHomoSapiensAlu]) await.
	
	bench r: (randomnessActor adoptObject: (ranClass to: 1)).
	bench randomStreamFact: (randomnessActor adoptObject: BGRandomStreamFactory new).
	
	(benchActor evaluateAsync: [
		bench twoIUBAmbiguityCodes.
		bench threeHomoSapiensFrequency.]) await.
	
	out flush.
	
	^ out contents.! !

!ATBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 1/15/2012 22:26'!
benchATNBody
	| problemSize bodiesActor bodies benchActor result bodyClass |
	problemSize := self problemSize
			ifNotNil: [ self problemSize ]
			ifNil:    [ BGNBody defaultProblemSize ].

	bodiesActor := self createActor: 'Bodies'.
	benchActor  := self createActor: 'BenchExec'.
	
	bodyClass := (Smalltalk hasClassNamed: #OstBGNBody)
		ifTrue: [Smalltalk at: #OstBGNBody]
		ifFalse:[BGNBody].
	
	bodies := bodiesActor adoptObject: bodyClass new.
	
	result := benchActor evaluateAsync: [
		BGNBody nbody: problemSize with: bodies.].
	
	^ result read.
! !

!ATBenchmarkGameSuite methodsFor: 'tracing' stamp: 'StefanMarr 1/4/2012 13:44'!
createActor: name
	^ (ATActor new) name: name; yourself! !

ATBenchmarkGameSuite subclass: #ATBenchmarkGameSuiteTraced
	instanceVariableNames: 'actors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!ATBenchmarkGameSuiteTraced methodsFor: 'accessing' stamp: 'StefanMarr 1/3/2012 22:08'!
actors
	^ actors! !

!ATBenchmarkGameSuiteTraced methodsFor: 'tracing' stamp: 'StefanMarr 1/4/2012 13:43'!
createActor: name
	| actor |
	actor := ATTracingActor new.
	actor name: name.
	actors add: actor.
	^ actor! !

!ATBenchmarkGameSuiteTraced methodsFor: 'initialization' stamp: 'StefanMarr 1/3/2012 22:08'!
initialize
	super initialize.
	actors := OrderedCollection new.! !

TestCase subclass: #ATBenchmarkTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!ATBenchmarkTests methodsFor: 'tests' stamp: 'StefanMarr 1/3/2012 22:48'!
testATBinaryTrees
	| bench actors |
	bench := ATBenchmarkGameSuiteTraced new.
	self assert: -1 equals: bench benchATBinaryTrees.
	
	actors := bench actors.
	
	self assert: 2 equals: actors size.
	
	self assert: 2692 equals: (actors at: 1) numberOfProcessedMessages. "Tree actor"
	self assert: 1    equals: (actors at: 2) numberOfProcessedMessages. "Bench actor"
	! !

!ATBenchmarkTests methodsFor: 'tests' stamp: 'StefanMarr 1/10/2012 14:20'!
testATFannkuchRedux
	| bench actors |
	bench := ATBenchmarkGameSuiteTraced new.
	self assert: 4 equals: bench benchATFannkuchRedux.
	
	actors := bench actors.
	
	self assert: 2 equals: actors size.
	
	self assert: 312 equals: (actors at: 1) numberOfProcessedMessages. "Gen actor"
	self assert: 1    equals: (actors at: 2) numberOfProcessedMessages. "Bench actor"
	! !

!ATBenchmarkTests methodsFor: 'tests' stamp: 'StefanMarr 1/4/2012 14:41'!
testATFasta
	| atBench result actors |
	atBench := ATBenchmarkGameSuiteTraced new.
	result := atBench benchATFasta.
	
	self assert: result equals: BGFasta fasta.
	
	actors := atBench actors.
	
	self assert: 2 equals: actors size.
	
	self assert: 20010 equals: (actors at: 1) numberOfProcessedMessages. "randomness actor"
	self assert: 2 equals: (actors at: 2) numberOfProcessedMessages. "bench exec actor"
	! !

!ATBenchmarkTests methodsFor: 'tests' stamp: 'StefanMarr 1/3/2012 21:18'!
testATFloatLoop
	| bench actors |
	bench := ATBenchLoopsTraced new.
	self assert: #done equals: bench benchATFloatLoop.
	
	actors := bench actors.
	
	self assert: 1 equals: actors size.
	self assert: 1 equals: (actors at: 1) numberOfProcessedMessages.! !

!ATBenchmarkTests methodsFor: 'tests' stamp: 'StefanMarr 1/4/2012 10:57'!
testATIntLoop
	| bench actors |
	bench := ATBenchLoopsTraced new.
	self assert: #done equals: bench benchATIntLoop.
	actors := bench actors.
	
	self assert: 1 equals: actors size.
	self assert: 1 equals: (actors at: 1) numberOfProcessedMessages. "loop actor"! !

!ATBenchmarkTests methodsFor: 'tests' stamp: 'StefanMarr 1/4/2012 14:38'!
testATNBody
	| bench result actors bodyMessageCnt execMsgCnt |
	bench := ATBenchmarkGameSuiteTraced new.
	result := bench benchATNBody.
	
	self assert: (((result at: 2) asString) beginsWith: '-0.169087605').
	
	actors := bench actors.
	
	self assert: 2 equals: actors size.
	
	bodyMessageCnt := (actors at: 1) numberOfProcessedMessages.
	execMsgCnt := (actors at: 2) numberOfProcessedMessages.
	
	self assert: (bodyMessageCnt = (BGNBody defaultProblemSize + 2) or: [
					 bodyMessageCnt = (BGNBody defaultProblemSize + 3)])
	     description: 'Minimal differences are possible because of the different semantics'.
	
	self assert: 1 equals: execMsgCnt. "benchmark exec actor"! !

!ATBenchmarkTests methodsFor: 'tests' stamp: 'StefanMarr 1/3/2012 22:26'!
testLocalSend
	| bench actors result |
	bench := ATMicroBenchmarksTraced new.
	result := bench benchLocalSend.
	
	self assert: ATMicroBenchmarks defaultProblemSize
	     equals: result.
	
	actors := bench actors.
	
	self assert: 1 equals: actors size.
	
	self assert: 1 equals: (actors at: 1) numberOfProcessedMessages.! !

!ATBenchmarkTests methodsFor: 'tests' stamp: 'StefanMarr 3/31/2012 17:57'!
testRandomStreamActors
	| randomnessActor benchActor suite streamFact promise |
	suite := ATBenchmarkGameSuite new.
	randomnessActor := suite createActor: 'Randomness'.
	benchActor      := suite createActor: 'BenchExec'.
	
	streamFact := randomnessActor adoptObject: BGRepeatStreamFactory new.
	
	promise := benchActor evaluateAsync: [ | stream |
		stream := streamFact to: 2000
            on:'GGCCGGGCGCGGTGGCTCACGCCTGTAATCCCAGCACTTTGG',
               'GAGGCCGAGGCGGGCGGATCACCTGAGGTCAGGAGTTCGAGA',
               'CCAGCCTGGCCAACATGGTGAAACCCCGTCTCTACTAAAAAT',
               'ACAAAAATTAGCCGGGCGTGGTGGCGCGCGCCTGTAATCCCA',
               'GCTACTCGGGAGGCTGAGGCAGGAGAATCGCTTGAACCCGGG',
               'AGGCGGAGGTTGCAGTGAGCCGAGATCGCGCCACTGCACTCC',
               'AGCCTGGGCGACAGAGCGAGACTCCGTCTCAAAAA'.
		stream next. ].
	
	self assert: $G equals: promise read.! !

!ATBenchmarkTests methodsFor: 'tests' stamp: 'StefanMarr 1/3/2012 22:48'!
testRemoteSend
	| bench actors result |
	bench := ATMicroBenchmarksTraced new.
	result:= bench benchRemoteSend.
	
	self assert: ATMicroBenchmarks defaultProblemSize
	     equals: result.
	
	actors := bench actors.
	
	self assert: 2 equals: actors size.
	
	self assert: 1          equals: (actors at: 1) numberOfProcessedMessages. "counting actor"
	self assert: result + 1 equals: (actors at: 2) numberOfProcessedMessages. "counter actor"
	! !

!ATBenchmarkTests methodsFor: 'tests' stamp: 'StefanMarr 1/3/2012 22:48'!
testRemoteSendWithManyArguments
	| bench result actors |
	bench := ATMicroBenchmarksTraced new.
	result:= bench benchRemoteSendWithManyArguments.
	
	self assert: ATMicroBenchmarks defaultProblemSize
	     equals: result.
	
	
	actors := bench actors.
	
	self assert: 2 equals: actors size.
	
	self assert: 1          equals: (actors at: 1) numberOfProcessedMessages. "counting actor"
	self assert: result + 1 equals: (actors at: 2) numberOfProcessedMessages. "counter actor"
	! !

SMarkSuite subclass: #ATBenchmarks
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!ATBenchmarks methodsFor: 'tracing' stamp: 'StefanMarr 1/4/2012 14:28'!
createActor: name
	^ ATActor new name: name; yourself! !

ATBenchmarks subclass: #ATMicroBenchmarks
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!ATMicroBenchmarks class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/2/2012 17:52'!
defaultProblemSize
	^ 10000! !

!ATMicroBenchmarks methodsFor: 'benchmarking' stamp: 'StefanMarr 1/4/2012 13:45'!
benchLocalSend
	"In the omni version this one should be sensitive to field updates which will probably be measurable"
	| actor result |
	actor := self createActor: 'BenchExec'.
	
	result := actor evaluateAsync: [
		| o |
		o := ATMicroSubject new.
		1 to: self problemSize do: [:i|
			o incCounter.
		].
		o counter
	].

	^ result read! !

!ATMicroBenchmarks methodsFor: 'benchmarking' stamp: 'StefanMarr 1/4/2012 13:45'!
benchRemoteSend
	| actor cntActor cnt result |
	actor := self createActor: 'BenchExec'.
	cntActor := self createActor: 'Counter'.
	
	cnt := cntActor adoptObject: ATMicroSubject new.
	
	result := actor evaluateAsync: [
		1 to: self problemSize do: [:i|
			cnt incCounter.
		].
		cnt counter
	].

	^ result read! !

!ATMicroBenchmarks methodsFor: 'benchmarking' stamp: 'StefanMarr 1/4/2012 13:45'!
benchRemoteSendWithManyArguments
	| actor cntActor cnt result |
	actor := self createActor: 'BenchExec'.
	cntActor := self createActor: 'Counter'.
	
	cnt := cntActor adoptObject: ATMicroSubject new.
	
	result := actor evaluateAsync: [
		1 to: self problemSize do: [:i|
			cnt process: i a: i b: i c: i d: i e: i f: i g: i h: i i: i.
		].
		cnt counter
	].

	^ result read! !

ATMicroBenchmarks subclass: #ATMicroBenchmarksTraced
	instanceVariableNames: 'actors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!ATMicroBenchmarksTraced methodsFor: 'accessing' stamp: 'StefanMarr 1/3/2012 22:23'!
actors
	^ actors! !

!ATMicroBenchmarksTraced methodsFor: 'tracing' stamp: 'StefanMarr 1/4/2012 14:28'!
createActor: name
	| actor |
	actor := ATTracingActor new.
	actor name: name.
	actors add: actor.
	^ actor! !

!ATMicroBenchmarksTraced methodsFor: 'initialization' stamp: 'StefanMarr 1/3/2012 22:23'!
initialize
	super initialize.
	actors := OrderedCollection new.! !

OstObject subclass: #ATMicroSubject
	instanceVariableNames: 'cnt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!ATMicroSubject methodsFor: 'accessing' stamp: 'StefanMarr 1/2/2012 14:32'!
counter
	^ cnt! !

!ATMicroSubject methodsFor: 'testing helpers' stamp: 'StefanMarr 1/2/2012 14:31'!
incCounter
	cnt := cnt + 1.! !

!ATMicroSubject methodsFor: 'initialization' stamp: 'StefanMarr 1/2/2012 14:31'!
initialize
	super initialize.
	cnt := 0.! !

!ATMicroSubject methodsFor: 'testing helpers' stamp: 'StefanMarr 1/2/2012 14:41'!
process: z a: a b: b c: c d: d e: e f: f g: g h: h i: i
	cnt := cnt + 1.! !

OstObject subclass: #BGOmniFacory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

BGOmniFacory subclass: #BGRepeatStreamFactory
	instanceVariableNames: 'clsInstance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

BGRepeatStreamFactory subclass: #BGRandomStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!BGRandomStreamFactory methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/4/2012 12:52'!
initialize
	super initialize.
	clsInstance := BGRandomStream.! !

!BGRepeatStreamFactory methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/4/2012 12:49'!
initialize
	super initialize.
	clsInstance := BGRepeatStream.! !

!BGRepeatStreamFactory methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/4/2012 12:48'!
to: anInteger on: aCollection
	^ clsInstance to: anInteger on: aCollection! !

BGOmniFacory subclass: #BGTreeNodeFactory
	instanceVariableNames: 'clsInstance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!BGTreeNodeFactory methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/4/2012 15:10'!
bottomUpTree: anItem depth: anInteger
	^ clsInstance bottomUpTree: anItem depth: anInteger.! !

!BGTreeNodeFactory methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/15/2012 22:08'!
initialize
	super initialize.
	
	(Smalltalk hasClassNamed: #OstBGTreeNode)
		ifTrue:  [clsInstance := Smalltalk at: #OstBGTreeNode]
		ifFalse: [clsInstance := BGTreeNode].
	
	! !

SMarkLoops subclass: #ATBenchLoops
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!ATBenchLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 10:45'!
benchATArrayAccess
	| promise actor |
	
	actor := self createActor: 'BenchExec'.
	promise := actor evaluateAsync: [
		self benchArrayAccess.
		#done
	].

	^ promise read! !

!ATBenchLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 10:45'!
benchATClassVarBinding
	| promise actor |
	
	actor := self createActor: 'BenchExec'.
	promise := actor evaluateAsync: [
		self benchClassVarBinding.
		#done
	].

	^ promise read! !

!ATBenchLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/4/2012 13:45'!
benchATFloatLoop
	| promise actor |
	
	actor := self createActor: 'BenchExec'.
	promise := actor evaluateAsync: [
		self benchFloatLoop.
		#done
	].

	^ promise read! !

!ATBenchLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 10:46'!
benchATInstVarAccess
	| promise actor |
	
	actor := self createActor: 'BenchExec'.
	promise := actor evaluateAsync: [
		self benchInstVarAccess.
		#done
	].

	^ promise read! !

!ATBenchLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/4/2012 13:46'!
benchATIntLoop
	| promise actor |
	
	actor := self createActor: 'BenchExec'.
	promise := actor evaluateAsync: [
		self benchIntLoop.
		#done
	].

	^ promise read! !

!ATBenchLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 10:46'!
benchATSend
	| promise actor |
	
	actor := self createActor: 'BenchExec'.
	promise := actor evaluateAsync: [
		self benchSend.
		#done
	].

	^ promise read! !

!ATBenchLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 10:46'!
benchATSendWithManyArguments
	| promise actor |
	
	actor := self createActor: 'BenchExec'.
	promise := actor evaluateAsync: [
		self benchSendWithManyArguments.
		#done
	].

	^ promise read! !

!ATBenchLoops methodsFor: 'tracing' stamp: 'StefanMarr 1/4/2012 14:26'!
createActor: name
	^ ATActor new name: name; yourself! !

ATBenchLoops subclass: #ATBenchLoopsTraced
	instanceVariableNames: 'actors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AmbientTalkST-Benchmarks'!

!ATBenchLoopsTraced methodsFor: 'accessing' stamp: 'StefanMarr 1/3/2012 20:36'!
actors
	^ actors! !

!ATBenchLoopsTraced methodsFor: 'tracing' stamp: 'StefanMarr 1/4/2012 14:26'!
createActor: name
	| actor |
	actor := ATTracingActor new.
	actor name: name.
	actors add: actor.
	^ actor! !

!ATBenchLoopsTraced methodsFor: 'initialization' stamp: 'StefanMarr 1/3/2012 20:36'!
initialize
	super initialize.
	actors := OrderedCollection new.! !
